﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDataModels.Enums;
using PlumbingRepairDataModels.Models;
using System.ComponentModel.DataAnnotations;

namespace PlumbingRepairDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }

        public int WorkId { get; private set; }

        public string WorkName { get; private set; } = string.Empty;
        [Required]
        public int ClientId { get; private set; }

		public int? ImplementerId { get; set; }
		[Required]
		public int Count { get; private set; }

        [Required]
        public double Sum { get; private set; }

        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public virtual Work Work { get; set; }
        public virtual Client Client { get; set; }
		public virtual Implementer? Implementer { get; set; }

		public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Order()
            {
                Id = model.Id,
                WorkId = model.WorkId,
                WorkName = model.WorkName,
                ClientId = model.ClientId,
				ImplementerId = model.ImplementerId,
				Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Status = model.Status;
            DateImplement = model.DateImplement;
			ImplementerId = model.ImplementerId;
		}

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            WorkId = WorkId,
            WorkName = Work.WorkName,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
			ImplementerId = ImplementerId,
			ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty,
			Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
